package at.ac.uibk.gitsearch.service;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.repository.search.testESService.ElasticSearchTestConfiguration;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.concurrent.TimeUnit;
import org.codeability.sharing.plugins.api.search.util.ExerciseId;
import org.elasticsearch.node.NodeValidationException;
import org.gitlab4j.api.GitLabApiException;
import org.junit.Assert;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.Timeout;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.util.StreamUtils;

@SpringBootTest(classes = GitsearchApp.class)
@WithMockUser(value = "test", authorities = "sharing")
public class GitLabServiceIT {

    @Autowired
    GitlabService gitlabService;

    final String publicGitLabProjectId = "318";

    @Autowired
    ElasticSearchTestConfiguration elasticSearchTestConfiguration;

    static ElasticSearchTestConfiguration staticElasticSearchTestConfiguration;

    @BeforeEach // must be started as BeforeEach, in order to autowire the ElasticSearchTestConfiguration
    @Timeout(value = 2, unit = TimeUnit.MINUTES)
    public void setUpESServer() throws IOException, NodeValidationException {
        if (staticElasticSearchTestConfiguration == null) {
            elasticSearchTestConfiguration.startTestNode();
            staticElasticSearchTestConfiguration = elasticSearchTestConfiguration;
        }
    }

    @AfterAll
    public static void shutDownESServer() {
        if (staticElasticSearchTestConfiguration != null) {
            staticElasticSearchTestConfiguration.stopTestContainer();
            staticElasticSearchTestConfiguration = null;
        }
    }

    @Test
    public void testGitLabAccess() {
        Assert.assertTrue(gitlabService.repositoryExists(publicGitLabProjectId));
        Assert.assertFalse(gitlabService.repositoryExists("XXXX--keineId--XXX"));
    }

    @Test
    public void testGitLabDownload() throws GitLabApiException, IOException {
        final InputStream repositoryZip = gitlabService.getRepositoryZip(publicGitLabProjectId);
        // schwacher Test :-(
        Assert.assertNotNull(repositoryZip);
    }

    @Test
    public void getREADMEmd() throws GitLabApiException, IOException {
        final InputStream repositoryFile = gitlabService.getRepositoryFile(new ExerciseId("318", null), "README.md");
        Assert.assertNotNull(repositoryFile);

        ByteArrayOutputStream baos = new ByteArrayOutputStream();

        StreamUtils.copy(repositoryFile, baos);

        Assert.assertTrue("At least 50 characters?", baos.size() > 50);
    }
}
